#ifndef CLOGGER_H_INCLUDED_C1D2C1FC
#define CLOGGER_H_INCLUDED_C1D2C1FC
#include <errorhandling/envdebug.h>
#include "RACE_Dumpable.h"
#include "RACE_Accumulator.h"
#include "RACE_Synch_Task.h"
#include "RACE_LogSwitchingStrategy.h"
#include "RACE_LoggerStrategy.h"

//##Documentation
//## The Logger schedules the writing of Dumpables in a Logfile.
//##
//## The Logger has a internaly Queue and a Thread which
//## consumes it (Active Object pattern)
//##

class RACE_Logger : public RACE_LogSwitcher, public RACE_Synch_Task
{
 public:
  RACE_Logger(RACE_WriterInterface *logfile,RACE_LoggerStrategy *strat,
              RACE_LogSwitchingStrategy *lswstat=0);
  const char *GetName();
  //##Documentation
  //## schedules the a Dumpable in the logger
  void Log(RACE_Dumpable *dumpable);

  //##Documentation
  //## It cancels the Excecution of a Logscript
  //## This may be otherwritten in Subclasses,
  //## but this function must be called
  virtual void Cancel();

  //##Documentation
  //## Flushes the internal buffer
  void Flush();
  RACE_Logger *Duplicate();
  void Release();

  RACE_LogSwitchingStrategy *GetSwitchingStrategy() { return mp_LogSwitchingStrategy; }

  //##Documentation
  //## Abstract callback for notification of logger thread stop.
  virtual void logger_thread_stopped();

 public:
  //##Documentation
  //## the thread from RACE_Task
  virtual int svc(void);
  //##Documentation
  //## from the RACE_LogSwitcher-Interface
  virtual void SwitchLog();
  //##Documentation
  //## from the RACE_LogSwitcher-Interface
  virtual void InfoSnapshotWritten();

 protected:
  ~RACE_Logger();
  RACE_WriterInterface *mp_logfile;
  RACE_Accumulator m_accu;
  RACE_LogSwitchingStrategy *mp_LogSwitchingStrategy;
  bool m_canceled;
  RACE_LoggerStrategy *m_strat;
  int m_refcount;
  static envdebug *ms_debug;
};


#endif /* CLOGGER_H_INCLUDED_C1D2C1FC */

