#include "RACE_LogfileOffset.h"
#include "RACE_Accumulator.h"
#include <errorhandling/envdebug.h>

#include <string.h>
#include <iostream>

envdebug* debug = new envdebug ("accutest");

unsigned generated_fragmented = 0;
unsigned generated_unfragmented = 0;
unsigned notified = 0;

//##Documentation
//##  A Dumpable for testing

class RACE_MyDumpable : public RACE_Dumpable
{
  char pat_;
  bool frag;
  RACE_MyDumpable(char pat):pat_(pat) {} // For Cloning
  RACE_LogfileOffset dumpable_position;
  bool dumpable_position_valid;
public:
  RACE_MyDumpable(size_t s,char pat,bool f = false)
    :RACE_Dumpable(s),pat_(pat),frag(f),dumpable_position(0,0),dumpable_position_valid(false)
  {
    debug->print ("%p create %sfragmented instance of RACE_MyDumpable", this, (frag ? "" : "un"));
    memset(wr_ptr(),pat,s);
    wr_ptr(s);
  }
  virtual void InformOffset(RACE_LogfileOffset offset)
  {
    if (dumpable_position_valid) {
      ERRLOG << "RACE_MyDumpable::InformOffset: dumpable_position is still valid" << std::endl;
      abort ();
    }
    dumpable_position = offset;
    dumpable_position_valid = true;
    std::cout << "Informed Pattern '" << pat_ << "'" << std::endl;
  }
  virtual void InformCompleted()
  {
    if (! dumpable_position_valid) {
      ERRLOG << "RACE_MyDumpable::InformCompleted: dumpable_position is not valid" << std::endl;
      abort ();
    }
    notified++;
    std::cout << "Written Pattern '" << pat_ << "' at offset " << dumpable_position.part
              << ':' << dumpable_position.offset << std::endl;
    debug->print ("%p notified %u. on %sfragmented instance", this, notified, (frag ? "" : "un"));
    //debug->trace (5);
    dumpable_position_valid = false;
  }
protected:
  virtual ~RACE_MyDumpable () {}
  virtual RACE_Message_Block *CloneManagementData() const;
};

RACE_Message_Block *RACE_MyDumpable::CloneManagementData() const
{
  RACE_MyDumpable* ret = new RACE_MyDumpable(pat_);
  return ret;
}

RACE_Dumpable *genentry(size_t s,char pat)
{
  generated_unfragmented++;
  std::cout << "genentry(" << s << "," << pat << ");" << std::endl;
  return new RACE_MyDumpable(s,pat);
}

RACE_Dumpable *genfragmented(size_t s,char pat1,char pat2)
{
  generated_fragmented++;
  std::cout << "genfragmented(" << s << "," << pat1 << "," << pat2 << ");" << std::endl;
  RACE_Dumpable *f = new RACE_MyDumpable(s,pat1,true);
  f->expand();
  memset(f->wr_ptr(),pat2,s);
  f->wr_ptr(s);
  return f;
}

//##Documentation
//##  Simulates a Writer

class RACE_TestWriter : public RACE_WriterInterface
{
public:
  virtual void Close() {}
  virtual void Open() {}
  virtual void SwitchPart() {}
  virtual int WriteV(const  struct  iovec*  v,  int count);
  virtual int PartNo();
};

int RACE_TestWriter::WriteV(const  struct  iovec*  v,  int count)
{
  int i;
  std::cout << "count=" << count << std::endl;
  size_t sum=0;
  for(i=0;i<count;i++)
  {
    char *str = (char *)(v[i].iov_base);
    std::cout << v[i].iov_len << " " << str[0] << "  " ;
    sum+=v[i].iov_len;
  }
  std::cout << "\nsum=" << sum << std::endl;
  return sum;
}

int RACE_TestWriter::PartNo()
{
  return 0;
}

int main(int argc,char **argv)
{
  RACE_LOGMSGBACKEND::instance()->init(argc,argv);
  RACE_TestWriter log;
  RACE_Accumulator acc(&log);

  acc.Add(genentry(4096,'0'));
  acc.Add(genentry(400,'1'));
  acc.Add(genentry(1024,'2'));
  acc.Add(genentry(2048,'3'));
  acc.Add(genentry(1024,'4'));
  acc.Add(genentry(8192,'5'));
  acc.Add(genentry(80,'6'));
  acc.Flush();
  std::cout << "too many (30>20) litte blocks, need flushing ..." << std::endl;
  for(int i=0;i<30;i++)
  {
    acc.Add(genentry(5,'l'));
  }
  acc.Flush();
  std::cout << "Test with a fragmented Dumpable " << std::endl;
  acc.Add(genfragmented(1024,'a','b'));
  acc.Add(genentry(10,'-'));
  acc.Add(genfragmented(4096,'c','d'));
  acc.Flush();
  std::cout << "generated_fragmented:" << generated_fragmented << " generated_unfragmented:"
            << generated_unfragmented << " notified:" << notified << std::endl;
  if (notified == (generated_unfragmented + generated_fragmented)) {
    std::cout << "SUCCESS" << std::endl;
    return 0;
  }
  std::cout << "FAILURE" << std::endl;
  return 1;
}
