#include"RACE_LogfileReader.h"
#include<errorhandling/errorhandling.h>
#include<sys/types.h>
#include<sys/stat.h>
#include<fcntl.h>
#include<unistd.h>

static envdebug *debug = new envdebug("RACE_Logfile_Reader");
//using namespace RACE;



RACE::LogFileReader::LogFileReader(RACE::Interpreter *interp,RACE::FullBufferHandlingStrategy *fbhs)
  : BlockReader(),
    m_logtype(LOGTYPE_LOGFILE),fd_(-1), m_sizeofcurrentpart(0),
    m_IsEndofLog(false),
    m_pos(), m_wantedPart(0), buffer_(20,this,interp,fbhs)
{
  memset(logfilename_, '\0', sizeof(logfilename_));
}

RACE::LogFileReader::~LogFileReader()
{
}

//-----------------------------------------------------------
bool RACE::LogFileReader::Open(const char *logfilename)
{
  envdebug::component c(debug,"RACE::LogFileReader::Open");
  m_wantedPart=PART_UNDEFINED;
  m_pos.part=1;
  const size_t maxlen = sizeof(logfilename_) - 1;
  if (strlen(logfilename) > maxlen)
  {
    RACE_ERROR("logfile name is too long");
    return false;
  }
  strcpy(logfilename_, logfilename);
  char *pz = strrchr(logfilename_, '.');
  if (pz == NULL)
  {
    RACE_ERROR("malformed logfile name");
    return false;
  }
  if (strcmp(pz, ".log") == 0)
  {
    char *pe=pz;
    // We are reading backward ...
    m_wantedPart=0;
    int factor=1;
    pz--;
    while(isdigit(*pz))
    {
      m_wantedPart += (*pz-'0')*factor;
      factor*=10;
      pz--;
    }
    if(*pz=='_') // before the part number is '_'
    {
      *pz='\0';
    }
    else
    {
      // without '_' we have parst the seconds as part number, correct it
      *pe='\0';
      m_wantedPart=PART_UNDEFINED;
    }
  }
  else if (strcmp(pz, ".dp") == 0) // tracc Depository
  {
    *pz='\0';
    m_logtype = LOGTYPE_DEPOSITORY;
    m_wantedPart = PART_DEPOSITORY;
    pz--;
    /*
     * if a number is given, we want to read the old Versions of the logfiles
     * Without a number, the main-Depository must be read
     */
    if(isdigit(*pz))
    {
      m_wantedPart=PART_UNDEFINED;  // We want to read all *_
      while(isdigit(*pz))
      {
        pz--;
      }
      *pz='\0';
    }
  }
  debug->print("RACE::LogFileReader::Open: corrected basename=%s wantedpart=%d",logfilename_,m_wantedPart);
  m_pos.part=(m_wantedPart==PART_UNDEFINED ? 1 : m_wantedPart);
  m_pos.offset=0;

  snprintf(logfileindexname_,256,"%s.idx",logfilename_);
  return Openpart();
}


bool RACE::LogFileReader::Openpart()
{
  size_t filenamelen=strlen(logfilename_)+16;
  char *filename=(char *)alloca(filenamelen);
  RACE_COND_FAIL(filename==NULL,"%s:%d: alloca returns NULL",__FILE__,__LINE__);
  switch(m_logtype)
  {
    case LOGTYPE_LOGFILE:
      snprintf(filename,filenamelen,"%s_%d.log",logfilename_,m_pos.part);
      break;
    case LOGTYPE_DEPOSITORY:
      {
        if(m_pos.part==PART_DEPOSITORY)
          snprintf(filename,filenamelen,"%s.dp",logfilename_);
        else
          snprintf(filename,filenamelen,"%s_%d.dp",logfilename_,m_pos.part);
      }
      break;
    default:
      RACE_FATAL_ERROR("%s:%d: Unknown Logfiletype ",__FILE__,__LINE__);
  }
  return OpenCurrent(filename);
}

RACE_Dumpable *RACE::LogFileReader::Read() throw (RACE::ReadError)
{
  RACE_Dumpable *ret=NULL;
  int retry=0;
  if(fd_ < 0 && m_IsEndofLog)
  {
    if(m_wantedPart != PART_UNDEFINED) m_pos.part=m_wantedPart;
    if(!Openpart())
    {
      m_IsEndofLog=true;
      return NULL;
    }
  }
  do
  {
    retry=0;
    ret=ReadCurrent();
    if (m_IsEndofLog)
    {
      if (m_wantedPart == PART_UNDEFINED)
      {
        m_pos.part++;
        m_pos.offset = 0;
        // continue with the Next part
      }
      else
      {
        // the wanted part is read and we are finished
        return NULL;
      }
      if(Openpart())
      {
        debug->print("RACE::LogFileReader::Next: switch to part %d",m_pos.part);
        m_IsEndofLog=false;
        retry=1;
      }
   }
  }  while(retry);
  return ret;
}

RACE_LogfileOffset RACE::LogFileReader::GetPosition()
{
  return m_pos;
}

bool RACE::LogFileReader::OpenCurrent(const char *filename)
{
  fd_=::open(filename,O_RDONLY);
  if(fd_>=0)
  {
    struct stat statbuf;
    int r;
    r=::fstat(fd_,&statbuf);
    RACE_COND_FAIL(r<0,"RACE::LogFileReader::OpenCurrent: stat(%s) failed: %s",
                      filename,strerror(errno));
    m_sizeofcurrentpart = statbuf.st_size;
    m_IsEndofLog=false;
    buffer_.seekcorrection(0);
  }
  else
  {
    m_IsEndofLog=true;
    m_sizeofcurrentpart=0;
  }

  return (!m_IsEndofLog);
}

RACE_Dumpable *RACE::LogFileReader::ReadCurrent() throw (ReadError)
{
  RACE_Dumpable *ret=NULL;
  ret=buffer_.getDumpable();
  m_pos.offset=buffer_.getoffset();
  m_IsEndofLog=(ret==NULL);
  return ret;
}

void RACE::LogFileReader::CloseCurrent()
{
  if(fd_)
  {
    ::close(fd_);
    fd_=-1;
  }
}

void RACE::LogFileReader::Seek(RACE_LogfileOffset offset) throw (ReadError)
{
  if(offset.part==0 && offset.offset==0) // Begin or uninitialized
  {
    if(m_wantedPart != PART_UNDEFINED)
    {
      offset.part=m_wantedPart;
    }
    else
    {
      offset.part=1; //point to the first Data Part
    }
  }
  if(offset.part==PART_DEPOSITORY)
  {
    // In Depositories are all Logfileoffset in the current part
    offset.part = m_pos.part;
  }
  if(m_pos.part != offset.part)
  {
    m_pos.part=offset.part;
    bool r=Openpart();
    if(!r)
    {
      throw ReadError(offset,"Part not exist");
    }
  }
  // fseek let the size of the file grow - we have to control this
  // It is allowed to position at the end of the Part (> , not >= !!!)
  if(offset.offset>m_sizeofcurrentpart)
  {
    RACE_ERROR("LogFileReader::Seek: offset=%d > %ld=sizeofcurrentpart",offset.offset,m_sizeofcurrentpart);
    throw ReadError(offset,"Postion don't exist");
  }
  if(::lseek(fd_,offset.offset,SEEK_SET)<0)
  {
    RACE_ERROR("RACE::LogFileReader::Seek: fSeek(%d,SEEK_SET):%s",offset.offset,
                  strerror(errno));
    throw ReadError(offset,"Postion don't exist");
  }
  m_pos=offset;
  buffer_.seekcorrection(offset.offset);
}

RACE_Dumpable *RACE::LogFileReader::ReadAt(RACE_LogfileOffset offset) throw (ReadError)
{
  Seek(offset);
  return ReadCurrent();
}


size_t RACE::LogFileReader::ReadBlock(octet_t *puffer)
{
  return ::read(fd_,puffer,4096);
}

void RACE::LogFileReader::CorrectPos(off_t rel)
{
  lseek(fd_,rel,SEEK_CUR);
}

