#ifndef RACE_READBUFFER_H
#define RACE_READBUFFER_H
#include<RAMS/elementartypes.h>
#include<ace/Synch_T.h>
#include<RACE_Data_Block.h>
#include<RACE_ReadError.h>
#include<RACE_Interpreter.h>

namespace RACE
{
  typedef long Index_t;
  typedef unsigned long Count_t;

  class BlockReader
  {
  public:
    BlockReader(size_t bs=getpagesize()):blocksize_(bs) {}
    virtual RACE_LogfileOffset GetPosition()=0;
    virtual size_t ReadBlock(octet_t *puffer)=0;
    virtual void CorrectPos(off_t rel)=0;
    size_t getBlocksize() { return blocksize_; }
  protected:
    size_t blocksize_;
  };

  class ReadBuffer;

  //##Documentation
  //## A Exception Class
  class BufferFull {};

  //##Documentation
  //## controls the behavior when the ReadBuffer is full
  //## Note: depended on the realisation, more than one thread may call
  //## different Callbacks
  class FullBufferHandlingStrategy
  {
  public:
    FullBufferHandlingStrategy(ReadBuffer *rb=0):readbuffer_(rb) {}
    virtual ~FullBufferHandlingStrategy();
    void SetReadBuffer(ReadBuffer *rb) { readbuffer_=rb; }
    //##Documentation
    //## This is called from ReadBuffer::nextpage()
    //## When the situation is handeld, it returns an Index
    //## of the freepage
    virtual Index_t notifyBufferIsFull() throw (BufferFull);
    virtual void notifyPageGetFree(Index_t freepage);
  protected:
    ReadBuffer *readbuffer_;
  };

  class ReadBuffer : public RACE_Data_Block_Factory
  {
  public:
    struct Pagedescriptor
    {
      Index_t pageno;
      Count_t count;
      Pagedescriptor():pageno(0),count(0) {}
    };
    ReadBuffer(Index_t pagecount,BlockReader *br,Interpreter *interp,FullBufferHandlingStrategy *fbhs=0);
    void SetInterpreter(RACE::Interpreter *ipr) { interp_=ipr; }
    virtual ~ReadBuffer();
    virtual RACE_Data_Block *generate(size_t) throw (NoMoreDynamicMemory);
    //##Documentation
    //## MAY CALLED from an other Thread
    virtual void recycle(RACE_Data_Block *schrott);
    //##Documentation
    //## We need to count the accessors on a BufferPage
    //## MAY CALLED from an other Thread
    virtual void informduplicate(RACE_Data_Block *db);
    //##Documentation
    //## We need to count the accessors on a BufferPage
    //## MAY CALLED from an other Thread
    virtual void informrelease(RACE_Data_Block *db);
    //##Documentation
    //## if we have to wrap or to jump other a unfree block
    //## we chain the datablocks and set fragmented==true;
    RACE_Dumpable *getDumpable();

    //##Documentation
    //## if we change the position in the Logfile, we have to begin
    //## on a new page and correct the offset.
    void seekcorrection(off_t newoffset);
    off_t getoffset() { return offset_; }
    void dumppagetable();
  protected:
    octet_t *currentpage() { return index2addr(freepage_); }
    octet_t *nextpage(bool &directsuccessor) throw (BufferFull);

    //##Documentation
    //## Mark all pages of the datablock
    //## Encapsule the concurreent used data
    void mark(RACE_Data_Block *b);
    //##Documentation
    //## Mark all pages of the datablock
    //## Encapsule the concurreent used data
    void unmark(RACE_Data_Block *b);
  protected:
    octet_t *index2addr(Index_t i);
    Index_t addr2index(octet_t *addr);
    //##Documentation
    //## Helperfunktion, does errorhandling, sezts blockuncomplete_
    size_t readablock(bool &pageisdirectsuccessor,bool ex=false) throw (BufferFull,ReadError);
  private:
    Index_t indexmax_;
    BlockReader *blockreader_;
    Interpreter *interp_;
    FullBufferHandlingStrategy *fullbufferhandling_;
    octet_t *puffer_;
    Pagedescriptor *pdtable_;
    Index_t freepage_;
    octet_t *currentdump_;
    off_t offset_;
    size_t rest_;
    bool blockuncomplete_;
    ACE_Thread_Mutex mutex_;
  };
};
#endif
