/**
 * @file mseg.h
 * @brief definiert Speichersegmente zur verwendung mit relativen Zeigern
 * @author <A href="http://www.infnet.verein.de/~rw/">Rudolf Weber</A> 2000
 * nach einer Idee 1995
 *
 * Wichtige Aenderung 12.2.2004: Segemtte haben eine grosse und keine Länge
 */
#ifndef MSEQ_H
#define MSEQ_H
#include<sys/types.h>
#include<RAMS/system_specific.h>
#ifndef NULL
#define NULL (void *)0;
#endif

namespace RAMS
{
  typedef long PageNo_t;
/**
 * \brief grundlegenes Speichersegment
 *
 * Das Speichersegment verwaltet nur ganz wenige Daten, Anfang, Ende und
 * Laenge<br>
 * Damit kann es als Grundlage fuer relative Zeiger dienen.<br>
 * Bemerkung: Diese Klasse ist Technologieneutral,
 * So kann ein beliebiger Puffer verwaltet werden, z.B:
 * <UL>
 *  <LI>mit read eingelesener Speicher
 *  <LI>mit recv empfangener Speicher
 *  <LI>mit mmap gemappter Speicher
 *  <LI>ein statisch oder dynamisch allokierter Speicher
 *  <LI>...
 * </UL>
 */
class Mseg
{
 protected:
   void *basis; /// anfang
   void *ende;  /// wie der name schon sagt
   size_t size_;  /// Laenge
   /// Setzen eines Segmentes fuer Iteratoren
   void setto(char *anfang,size_t lang)
     { basis=anfang;ende=anfang+lang; size_=lang;
     }
 public:
   /**
    * initialiiserung auf ein Nullsegemt
    */
   Mseg() { basis=NULL;ende=NULL;size_=0;}
   /**
    * initialisierung auf einen Zeiger
    */
    Mseg(char *anfang,size_t lang) { basis=anfang;ende=anfang+lang; size_=lang; }
    /**
     * Sometimes we need to initialize later in constuctores
     */
    void init(char *anfang,size_t lang) { setto(anfang,lang); }
   /**
    * Anfang des Segments
    */
   const void *begin() { return basis; }
   /**
    * Ende des segments
    */
   const void *end()   { return ende; }
   /**
    * Laenge bzw Groesse des Segments
    */
   size_t size()     { return size_;  }

   /**
    * Hilfsfunktion:
    * ermittelt offset von zeiger
    * Bedingung : begin()<=addr<end()
    */
   off_t offset(void *addr) { return (char *)addr - (char*)begin(); }

   /**
    * Hilfsfunktion zur Errmittling des Seitenindexes
    * Assume Pagesize 2^12=4096
    */
   PageNo_t pageno(void *addr) { return offset(addr)>>LPAGESIZE; }
   PageNo_t maxpageno()        { return size()>>LPAGESIZE; }
   void *   pageno2addr(PageNo_t pageno) { return (void *)((char *)begin()+(pageno<<LPAGESIZE)); }
};

}
#endif

