#include<errorhandling/errorhandling.h>
#include<iostream>
#include"reliablesegment.h"
#include<unistd.h>
#include<errno.h>
#include<stdlib.h>
#include<sys/types.h>
#include<sys/stat.h>
#include<fcntl.h>
#include<sys/mman.h>

static envdebug debug("RAMS_HVSEG_ReliableSegment");

int TwinblockStrategy::open(char *filename)
{
  f=::open(filename,O_RDWR);
  if(f>0) return 0;
  if(errno == ENOENT)
  {
    f=::open(filename,O_RDWR|O_CREAT,0600);
    if(f>0)
    {
      return 1;
    }
    else
    {
      RACE_FATAL_ERROR("Can't open %s",filename);
    }
  }
  return -1;
}
void TwinblockStrategy::close() { ::close(f); }
int TwinblockStrategy::nplex() { return 2; }
int TwinblockStrategy::fd(int) { return f; }
off_t TwinblockStrategy::offset(int i,int pageno)
{
  return (nplex()*pageno+i)*BLOCKSIZE;
}

uint32_t ReliableSegment::calc_checksum(octet_t *page)
{
  uint32_t* i;
  uint32_t ret = 0;
  for(i = (uint32_t *)page;i < (uint32_t *)(page+PAGESIZE);i++)
  {
    ret ^= *i;
  }
  return ret;
}

ReliableSegment::ReliableSegment(ReliableSegStrategy *s): PMSeg()
{
  unsigned int lang=maxblocks*PAGESIZE;
  strat=s;
  fd0=::open("/dev/zero",O_RDONLY);
  RACE_COND_FAIL(fd0 < 0,"open(/dev/zero) failed");
  char *puffer=(char *)mmap(NULL,lang,PROT_READ,MAP_PRIVATE,fd0,0);
  RACE_COND_FAIL(puffer==(char *)MAP_FAILED,"mmap(/dev/zero) failed");
  set(puffer,lang);
  for(int p=0;p<maxblocks;p++)
  {
    vdata[p].dirty=false;
  }
}

void ReliableSegment::write_page(int pageno)
{
  struct iovec iv[2];
  VerwData_t *vd = &(vdata[pageno].pverw);
  octet_t *p = (octet_t *)pageno2addr(pageno);
  vd->checksum = calc_checksum(p);
  time(&(vd->stamp));

  iv[0].iov_base=vd;iv[0].iov_len=sizeof(VerwData_t);
  iv[1].iov_base=p; iv[1].iov_len=PAGESIZE;
  for(int i=0;i< strat->nplex();i++)
  {
    int r=lseek(strat->fd(i),strat->offset(i,pageno),SEEK_SET);
    if(r<0) continue;
    writev(strat->fd(i),iv,2);
    fsync(strat->fd(i));
  }
  vdata[pageno].dirty=false;
  protect(pageno);
}

void ReliableSegment::read_page_optimistic(int pageno)
{
  struct iovec iv[2];
  VerwData_t *vd = &(vdata[pageno].pverw);
  octet_t *p = (octet_t *)pageno2addr(pageno);
  unprotect(pageno);
  iv[0].iov_base=vd;iv[0].iov_len=sizeof(VerwData_t);
  iv[1].iov_base=p; iv[1].iov_len=PAGESIZE;

  int ok=(lseek(strat->fd(0),strat->offset(0,pageno),SEEK_SET)>=0);
  ok = ok && ((size_t)readv(strat->fd(0),iv,2)==BLOCKSIZE);
  ok = ok && (calc_checksum(p) == vd->checksum);
  if(!ok)
  {
    read_page_reliable(pageno);
  }
  vdata[pageno].dirty=false;
  protect(pageno);
}

void ReliableSegment::read_page_reliable(int pageno)
{
  static char lpuffer[BLOCKSIZE];
  char *data = lpuffer+sizeof(VerwData_t);
  VerwData_t *vd = (VerwData_t *)lpuffer;
  unprotect(pageno);
  bool atleastone=false;
  bool badblockfound=false;
  for(int i=0;i<strat->nplex();i++)
  {
    int ok=(lseek(strat->fd(i),strat->offset(i,pageno),SEEK_SET)>=0);
    ok = ok && ((size_t)read(strat->fd(i),lpuffer,BLOCKSIZE)==BLOCKSIZE);
    ok = ok && (calc_checksum((octet_t*)data) == vd->checksum);
    if(!ok) { badblockfound=true; continue; }
    bool newer=false;
    if(!atleastone || (newer=(vd->stamp > vdata[pageno].pverw.stamp)) )
    {
      memcpy((char *)&(vdata[pageno].pverw),vd,sizeof(VerwData_t));
      memcpy((char *)begin()+pageno*PAGESIZE,data,PAGESIZE);
      badblockfound = badblockfound || newer;
      atleastone=true;
    }
  }
  RACE_COND_FAIL(!atleastone,"Fatal: can't read any block");
  if(badblockfound) write_page(pageno);
  vdata[pageno].dirty=false;
  protect(pageno);
}

void  ReliableSegment::handle(RAMS::PageNo_t pageno,void *addr)
{
  debug.print("Page %ld is becomming dirty ... ",pageno);
  vdata[pageno].dirty=true;
}

void ReliableSegment::save()
{
  debug.print("ReliableSegment::save()\n");
  for(int p=0;p<maxblocks;p++)
  {
    if(vdata[p].dirty)
    {
      debug.print("\tSaving page %d ...",p);
      write_page(p);
    }
  }
}






